@@ -19,8 +19,8 @@ gs_plugin_initialize (GsPlugin *plugin)
 {
 	GsPluginData *priv = gs_plugin_alloc_data (plugin, sizeof(GsPluginData));
 	priv->app_system = gs_app_new ("system");
-	gs_app_set_kind (priv->app_system, AS_APP_KIND_OS_UPGRADE);
-	gs_app_set_state (priv->app_system, AS_APP_STATE_INSTALLED);
+	gs_app_set_kind (priv->app_system, AS_COMPONENT_KIND_OPERATING_SYSTEM);
+	gs_app_set_state (priv->app_system, GS_APP_STATE_INSTALLED);
 }
 
 void
