@@ -87,7 +87,7 @@ search_done_cb (GObject *source,
 	g_variant_builder_init (&builder, G_VARIANT_TYPE ("as"));
 	for (i = 0; i < gs_app_list_length (list); i++) {
 		GsApp *app = gs_app_list_index (list, i);
-		if (gs_app_get_state (app) != AS_APP_STATE_AVAILABLE)
+		if (gs_app_get_state (app) != GS_APP_STATE_AVAILABLE)
 			continue;
 		g_variant_builder_add (&builder, "s", gs_app_get_unique_id (app));
 
@@ -107,7 +107,7 @@ gs_shell_search_provider_get_app_sort_key (GsApp *app)
 
 	/* sort available apps before installed ones */
 	switch (gs_app_get_state (app)) {
-	case AS_APP_STATE_AVAILABLE:
+	case GS_APP_STATE_AVAILABLE:
 		g_string_append (key, "9:");
 		break;
 	default:
@@ -117,7 +117,7 @@ gs_shell_search_provider_get_app_sort_key (GsApp *app)
 
 	/* sort apps before runtimes and extensions */
 	switch (gs_app_get_kind (app)) {
-	case AS_APP_KIND_DESKTOP:
+	case AS_COMPONENT_KIND_DESKTOP_APP:
 		g_string_append (key, "9:");
 		break;
 	default:
@@ -224,7 +224,6 @@ handle_get_result_metas (GsShellSearchProvider2	*skeleton,
 	GsShellSearchProvider *self = user_data;
 	GVariantBuilder meta;
 	GVariant *meta_variant;
-	GdkPixbuf *pixbuf;
 	gint i;
 	GVariantBuilder builder;
 
@@ -232,6 +231,7 @@ handle_get_result_metas (GsShellSearchProvider2	*skeleton,
 
 	for (i = 0; results[i]; i++) {
 		GsApp *app;
+		g_autoptr(GIcon) icon = NULL;
 		g_autofree gchar *description = NULL;
 
 		/* already built */
@@ -248,11 +248,20 @@ handle_get_result_metas (GsShellSearchProvider2	*skeleton,
 		g_variant_builder_init (&meta, G_VARIANT_TYPE ("a{sv}"));
 		g_variant_builder_add (&meta, "{sv}", "id", g_variant_new_string (gs_app_get_unique_id (app)));
 		g_variant_builder_add (&meta, "{sv}", "name", g_variant_new_string (gs_app_get_name (app)));
-		pixbuf = gs_app_get_pixbuf (app);
-		if (pixbuf != NULL)
-			g_variant_builder_add (&meta, "{sv}", "icon", g_icon_serialize (G_ICON (pixbuf)));
 
-		if (gs_utils_list_has_app_fuzzy (self->search_results, app) &&
+		/* ICON_SIZE is defined as 24px in js/ui/search.js in gnome-shell */
+		icon = gs_app_get_icon_for_size (app, 24, 1, NULL);
+		if (icon != NULL) {
+			g_autofree gchar *icon_str = g_icon_to_string (icon);
+			if (icon_str != NULL) {
+				g_variant_builder_add (&meta, "{sv}", "gicon", g_variant_new_string (icon_str));
+			} else {
+				g_autoptr(GVariant) icon_serialized = g_icon_serialize (icon);
+				g_variant_builder_add (&meta, "{sv}", "icon", icon_serialized);
+			}
+		}
+
+		if (gs_utils_list_has_component_fuzzy (self->search_results, app) &&
 		    gs_app_get_origin_hostname (app) != NULL) {
 			/* TRANSLATORS: this refers to where the app came from */
 			g_autofree gchar *source_text = g_strdup_printf (_("Source: %s"),
@@ -361,8 +370,8 @@ search_provider_dispose (GObject *obj)
 static void
 gs_shell_search_provider_init (GsShellSearchProvider *self)
 {
-	self->metas_cache = g_hash_table_new_full ((GHashFunc) as_utils_unique_id_hash,
-						   (GEqualFunc) as_utils_unique_id_equal,
+	self->metas_cache = g_hash_table_new_full ((GHashFunc) as_utils_data_id_hash,
+						   (GEqualFunc) as_utils_data_id_equal,
 						   g_free,
 						   (GDestroyNotify) g_variant_unref);
 
